
#ifndef rgrl_feature_face_region_h_
#define rgrl_feature_face_region_h_

//:
// \file
// \brief Class to represent a N-d face edge element.  It has 1 normal
//    direction and N-1 tangent directions.  
// \author Chuck Stewart
// \date 16 Sep 2003

#include <rgrl/rgrl_feature_face_pt.h>
#include <vcl_vector.h>

class rgrl_feature_face_region : public rgrl_feature_face_pt {
public:
  rgrl_feature_face_region( vnl_vector< double > const& location,
                            vnl_vector< double > const& normal );

  rgrl_feature_face_region( vnl_vector< double > const& location,
                            vnl_vector< double > const& normal,
                            double                      half_width,
                            double                      radius );

  rgrl_feature_type type() const;

  //:  Return the half_width ("fatness") of the surface (or the transition
  //   region)  
  double half_width() const { return half_width_; }

  //:  Return the radius on the tangent surface
  double radius() const { return radius_; }

  //: Result is a rgrl_feature_face_region, but the region is not transformed.
  rgrl_feature_sptr transform( rgrl_transformation const& xform ) const;

  //: Result is a rgrl_feature_face_region with a transformed region as well.
  rgrl_feature_sptr transform_region( rgrl_transformation const& xform ) const;

  //:  Extract the pixel coordinates within the oriented rectangular
  //   solid defined by the feature.
  vcl_vector< vnl_vector<int> > const& pixel_coordinates( vnl_vector< double > const& spacing_ratio );

private:

  //:
  // Create an uninitialized face_region of dimension dim
  //
  rgrl_feature_face_region();

  //:  The normal half width, and the radius on the tangent surface.
  double half_width_;
  double radius_;

  //  The pixel coordinates in the image are computed the first time
  //  they are needed and then cached.  This is safe the location of
  //  the feature does not change.
  bool pixel_coordinates_cached_;
  vcl_vector< vnl_vector<int> > pixel_coordinates_;
};


#endif
