#ifndef rgrl_feature_set_location_h_
#define rgrl_feature_set_location_h_

//:
// \file
// \author Amitha Perera
// \date   Feb 2003


//#include <rsdl/rsdl_bins.h>
#include <rsdl/rsdl_fwd.h>


#include "rgrl_feature.h"
#include "rgrl_feature_set.h"
#include "rgrl_scoped_ptr.h"

//: A set of features grouped only by N-d location.
//
// This class is used to store a collection of discrete features for
// which proximity is determined only by location. There is no
// concepts of "segments"; each feature is considered independent.
//
// The class is templated over the dimension in order to provide
// efficient implementation of the queries.
//
template<unsigned N>
class rgrl_feature_set_location
  : public rgrl_feature_set
{
public:
  rgrl_feature_set_location( feature_vector const& features );

  ~rgrl_feature_set_location();

  feature_vector
  features_in_region( rgrl_roi const& roi ) const;

  //: Nearest feature based on Euclidiean distance
  //
  rgrl_feature_sptr 
  nearest_feature( rgrl_feature_sptr feature ) const;

  //: Return all features within a given Euclidiean distance
  //
  feature_vector
  features_within_distance( rgrl_feature_sptr feature, double distance ) const;

  //:  Return the k nearest features based on Euclidiean distance.
  feature_vector
  k_nearest_features( rgrl_feature_sptr feature, unsigned int k ) const;

  //:  Return the type of feature
  rgrl_feature_type
  type() const;

private:
  typedef rsdl_bins<N,double,rgrl_feature_sptr> bin_type;

  rgrl_feature_type type_;
  rgrl_scoped_ptr< bin_type > bins_;
};


#endif // rgrl_feature_set_location_h_
