//:
// \file
// \author Amitha Perera
// \date   Feb 2003

#include "rgrl_feature_set_location.h"

#include <vcl_cassert.h>

#include <rsdl/rsdl_bins.h>
#include <rsdl/rsdl_bins.txx> // to avoid explicit instantiation

#include "rgrl_roi.h"


template<unsigned N>
rgrl_feature_set_location<N>::
rgrl_feature_set_location( feature_vector const& features )
{
  typedef typename bin_type::point_type point_type;

  assert( !features.empty() );

  // Determine the extents of the data. (And the type.)
  //
  point_type min;
  point_type max;
  feature_vector::const_iterator itr = features.begin();
  min = (*itr)->location();
  max = min;
  type_ = (*itr)->type();
  for( ; itr != features.end(); ++itr ) {
    vnl_vector<double> const& loc = (*itr)->location();
    assert( loc.size() == N );
    for( unsigned i=0; i < N; ++i ) {
      if( loc[i] < min[i] )    min[i] = loc[i];
      if( loc[i] > max[i] )    max[i] = loc[i];
    }
  }

  // Create the bins
  //
  point_type bin_sizes;
  bin_sizes.fill( 10.0 );
  bins_.reset( new bin_type( min, max, bin_sizes ) );

  // Add the data
  //
  for( feature_vector::const_iterator itr = features.begin();
       itr != features.end(); ++itr ) {
    bins_->add_point( (*itr)->location(), *itr );
  }
}


template<unsigned N>
rgrl_feature_set_location<N>::
~rgrl_feature_set_location()
{
}


template<unsigned N>
typename rgrl_feature_set_location<N>::feature_vector
rgrl_feature_set_location<N>::
features_in_region( rgrl_roi const& roi ) const
{
  feature_vector result;

  assert( roi.x0().size() == N );

  bins_->points_in_bounding_box( roi.x0(), roi.x1(), result );

  return result;
}


template<unsigned N>
rgrl_feature_sptr 
rgrl_feature_set_location<N>::
nearest_feature( rgrl_feature_sptr feature ) const
{
  feature_vector result = this->k_nearest_features( feature, 1 );
  assert( result.size() == 1 );
  return result[0];
}


template<unsigned N>
typename rgrl_feature_set_location<N>::feature_vector
rgrl_feature_set_location<N>::
features_within_distance( rgrl_feature_sptr feature, double distance ) const
{
  feature_vector results;
  bins_->points_within_radius( feature->location(), distance, results );
  return results;
}

  //:  Return the k nearest features based on Euclidiean distance.

template<unsigned N>
typename rgrl_feature_set_location<N>::feature_vector
rgrl_feature_set_location<N>::
k_nearest_features( rgrl_feature_sptr feature, unsigned int k ) const
{
  feature_vector results;
  bins_->n_nearest( feature->location(), k, results );
  return results;
}


template<unsigned N>
rgrl_feature_type
rgrl_feature_set_location<N>::
type() const
{
  return type_;
}
