//:
// \file
// \author Amitha Perera
// \date   Feb 2003

#include "rgrl_roi.h"

#include <vcl_cassert.h>

rgrl_roi::
rgrl_roi( unsigned dim )
  : x0_( dim, 0.0 ),
    x1_( dim, 0.0 )
{
}


rgrl_roi::
rgrl_roi( vnl_vector<double> const& x0, vnl_vector<double> const& x1 )
  : x0_( x0 ),
    x1_( x1 )
{
  assert( x0.size() == x1.size() );
}


vnl_vector<double> const&
rgrl_roi::
x0() const
{
  return x0_;
}


vnl_vector<double> const&
rgrl_roi::
x1() const
{
  return x1_;
}


void
rgrl_roi::
set_x0( vnl_vector<double> const& v )
{
  assert( v.size() == x0_.size() );
  x0_ = v;
}


void
rgrl_roi::
set_x1( vnl_vector<double> const& v )
{
  assert( v.size() == x1_.size() );
  x1_ = v;  
}

bool
rgrl_roi::
inside( vnl_vector<double> const& v ) const
{
  assert( v.size() == x1_.size() );
  bool inside = true;
  for (unsigned i =0; i<v.size(); ++i) {
    inside = inside && (x0_[i] <= v[i] && v[i] <= x1_[i]);
  }
  return inside;
}

bool 
rgrl_roi::
operator==( const rgrl_roi& other ) const
{
  return ( x0_ == other.x0_  &&
           x1_ == other.x1_ ); 
}

bool
rgrl_roi::
operator!=( const rgrl_roi& other ) const
{
  return !( *this == other );
}
