#ifndef rgrl_roi_h_
#define rgrl_roi_h_

//:
// \file
// \author Amitha Perera
// \date   Feb 2003
//
// \verbatim
// Modifications
// Sep 2003 Charlene Tsai added "inside" function
// \endverbatim
#include <vnl/vnl_vector.h>

//: Represents a region of interest.
//
// Currently, this is simply an axis aligned box. The coordinate
// dimensions depend on the specific estimation problem.
//
class rgrl_roi
{
public:
  //: Null initialization
  //
  // Initializes the region to the box from (0,0,...) to (0,0,...)
  //
  rgrl_roi( unsigned dimension );

  //: Initialize the region to a box from \a x0 to \a x1.
  rgrl_roi( vnl_vector<double> const& x0, vnl_vector<double> const& x1 );

  //: The lower coordinate of the box.
  vnl_vector<double> const& x0() const;

  //: The upper coordinate of the box.
  vnl_vector<double> const& x1() const;

  //: Set the lower coordinate of the box.
  void set_x0( vnl_vector<double> const& v );

  //: Set the upper coordinate of the box.
  void set_x1( vnl_vector<double> const& v );

  //: True if a point is inside the region
  bool inside( vnl_vector<double> const& v ) const;

  //: True iff all the components are equal.
  bool operator==( const rgrl_roi& other ) const;

  //: True iff some of the components are not equal.
  bool operator!=( const rgrl_roi& other ) const;

private:
  vnl_vector<double> x0_;
  vnl_vector<double> x1_;
};

#endif // rgrl_roi_h_
