
//:
// \file
// \brief Base class for transformation representation, estimations
// and application in generalized registration library  
// \author Chuck Stewart
// \date 15 Nov 2002

#include "rgrl_transformation.h"

#include <vcl_iostream.h>
#include <vcl_cassert.h>

rgrl_transformation::
~rgrl_transformation()
{
}


void
rgrl_transformation::
map_location( vnl_vector<double> const& from,
              vnl_vector<double>      & to    ) const
{
  map_loc( from, to );
}


vnl_vector<double>
rgrl_transformation::
map_location( vnl_vector<double> const& p ) const
{
  vnl_vector<double> result( p.size() );
  map_loc( p, result );
  return result;
}


void
rgrl_transformation::
map_direction( vnl_vector<double> const& from_loc,
               vnl_vector<double> const& from_dir,
               vnl_vector<double>      & to_dir    ) const
{
  map_dir( from_loc, from_dir, to_dir );
}


double
rgrl_transformation::
map_intensity( vnl_vector<double> const& /*from*/,
               double intensity ) const
{
  return intensity;
}

void 
rgrl_transformation::
inv_map( const vnl_vector<double>& to, 
         bool initialize_next,
         const vnl_vector<double>& to_delta,
         vnl_vector<double>& from,
         vnl_vector<double>& from_next_est) const
{
  vcl_cerr << "ERROR: rgrl_transformation::inv_map not defined\n";
  assert(0);
}

void 
rgrl_transformation::
inv_map( const vnl_vector<double>& to, 
         vnl_vector<double>& from ) const
{
  vcl_cerr << "ERROR: rgrl_transformation::inv_map not defined\n";
  assert(0);
}

rgrl_transformation_sptr
rgrl_transformation::
scale_by( double scale ) const
{
   vcl_cerr << "ERROR: rgrl_transformation::scale_by not defined\n";
   assert(0);
   return 0;
}
