#ifndef rgrl_view_gen_dbicp_h_
#define rgrl_view_gen_dbicp_h_

// \file
// \author Amitha Perera
// \date   Feb 2003

#include "rgrl_view_generator.h"

class rgrl_roi;

//: The dbicp view generator
//
// Grows the region based on the transfer error covariance
//
class rgrl_view_gen_dbicp
  : public rgrl_view_generator
{
public:
  //:
  rgrl_view_gen_dbicp( double max_growth_rate = 2, double min_growth_rate = 1, int max_num_update_global_region_per_model = -1);

  //: The next view is determined based on model selection and region growing
  virtual
  rgrl_view_sptr next_view( rgrl_view const&                        curr_view,
			    rgrl_set_of<rgrl_match_set_sptr> const& curr_matches,
			    rgrl_set_of<rgrl_scale_sptr> const&     curr_scales,
			    vcl_vector< rgrl_transformation_sptr >  const&  xforms,
                            vcl_vector<rgrl_estimator_sptr> const&  models,
                            vcl_vector<double > const &             errors);

private:
  double max_growth_rate_; //Growth factor
  double min_growth_rate_; //when the rate of region growing falls below min_growth_,
                           //the region is considered not grown, and stays the same.
  int max_num_update_global_region_; //in each model, the max number of updates for
                                  //the maximum registration region.
  int num_update_global_region_; //Keeps track of the number of update for the current model
};

#endif // rgrl_view_gen_dbicp_h_
