#ifndef rgrl_view_generator_h_
#define rgrl_view_generator_h_

// \file
// \brief  Generate the next "view" on the registration problem. 
// \author Chuck Stewart
// \date   25 Nov 2002

#include <vbl/vbl_ref_count.h>

#include "rgrl_set_of.h"
#include "rgrl_view_sptr.h"
#include "rgrl_match_set_sptr.h"
#include "rgrl_scale_sptr.h"
#include "rgrl_estimator_sptr.h"
#include "rgrl_weighter_sptr.h"
#include "rgrl_transformation_sptr.h"

#include "rgrl_view_generator_sptr.h"

//:
class rgrl_view_generator
  : public vbl_ref_count
{
public:
  virtual ~rgrl_view_generator();

  //:
  // Generate next view based on current view, matches, scale and
  // xform.  If the model changes, the transformation will change as
  // well.  If the resolution changes, the scales must be adjusted for
  // the new resolution, which may include new scales for new feature
  // types.
  //
  // It takes multiple match sets.
  virtual
  rgrl_view_sptr next_view( rgrl_view const&                        curr_view,
			    rgrl_set_of<rgrl_match_set_sptr> const& curr_matches,
			    rgrl_set_of<rgrl_scale_sptr> const&     curr_scales,
			    vcl_vector< rgrl_transformation_sptr >  const&  xforms,
                            vcl_vector<rgrl_estimator_sptr> const&  models,
                            vcl_vector<double > const &             errors) = 0;
  //: 
  // Generate next view.
  //
  // It takes single match set.
  virtual
  rgrl_view_sptr next_view( rgrl_view const&                        curr_view,
                            rgrl_match_set_sptr                     curr_matches,
                            rgrl_scale_sptr                         curr_scale,
                            vcl_vector< rgrl_transformation_sptr >  const&  xforms,
                            vcl_vector<rgrl_estimator_sptr> const&  models,
                            vcl_vector<double > const &             errors);
};

#endif // rgrl_view_generator_h_
